package cn.iocoder.yudao.module.bpm.service.oa;

import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.module.bpm.api.task.BpmProcessInstanceApi;
import cn.iocoder.yudao.module.bpm.api.task.dto.BpmProcessInstanceCreateReqDTO;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.vo.entry.BpmOAEntryCreateReqVO;
import cn.iocoder.yudao.module.bpm.convert.oa.BpmOAEntryConvert;
import cn.iocoder.yudao.module.bpm.dal.dataobject.oa.BpmOAEntryDO;
import cn.iocoder.yudao.module.bpm.dal.mysql.oa.BpmOAEntryMapper;
import cn.iocoder.yudao.module.bpm.enums.task.BpmProcessInstanceResultEnum;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.Map;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.bpm.enums.ErrorCodeConstants.OA_ENTRY_NOT_EXISTS;

/**
 * OA 入职申请 Service 实现类
 *
 * @author 符溶馨
 */
@Service
@Validated
public class BpmOAEntryServiceImpl implements BpmOAEntryService{

    /**
     * OA 入职申请对应的流程定义 KEY
     */
    public static final String PROCESS_KEY = "oa_entry";

    @Resource
    private BpmOAEntryMapper entryMapper;

    @Resource
    private BpmProcessInstanceApi processInstanceApi;
    
    @Override
    public Long createEntry(BpmOAEntryCreateReqVO createReqVO) {

        //插入OA 入职申请
        BpmOAEntryDO entry = BpmOAEntryConvert.INSTANCE.convert(createReqVO)
                .setResult(BpmProcessInstanceResultEnum.PROCESS.getResult());
        entryMapper.insert(entry) ;

        // 发起 BPM 流程
        Map<String, Object> processInstanceVariables = new HashMap<>();
        String processInstanceId = processInstanceApi.createProcessInstance(createReqVO.getUserId(),
                new BpmProcessInstanceCreateReqDTO().setProcessDefinitionKey(PROCESS_KEY)
                        .setVariables(processInstanceVariables).setBusinessKey(String.valueOf(entry.getId()))).getCheckedData();

        // 将工作流的编号，更新到 OA 入职申请单中
        entryMapper.updateById(new BpmOAEntryDO().setId(entry.getId()).setProcessInstanceId(processInstanceId));

        return entry.getId();
    }

    @Override
    public void updateEntryResult(Long id, Integer result) {

        validateLeaveExists(id);
        entryMapper.updateById(new BpmOAEntryDO().setId(id).setResult(result));
    }

    private void validateLeaveExists(Long id) {
        if (entryMapper.selectById(id) == null) {
            throw exception(OA_ENTRY_NOT_EXISTS);
        }
    }

    @Override
    public BpmOAEntryDO getEntry(Long id) {

        return entryMapper.selectById(id);
    }

    @Override
    public BpmOAEntryDO getEntryByOpenId(String openId) {

        return entryMapper.selectOne(new LambdaQueryWrapperX<BpmOAEntryDO>()
                .eq(BpmOAEntryDO::getOpenId, openId)
                .eq(BpmOAEntryDO::getResult, BpmProcessInstanceResultEnum.REJECT.getResult()));
    }
}
